/***
 * Fractal ADL Parser
 * Copyright (C) 2005-2006 STMicroelectronics, INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Ali Erdem Ozcan
 *
 * Contributor: Philippe Merle
 */

package org.objectweb.fractal.adl.bindings;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.AbstractLoader;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.Component;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.adl.interfaces.InterfaceContainer;
import org.objectweb.fractal.adl.types.TypeInterface;

/**
 * A {@link org.objectweb.fractal.adl.Loader} to check {@link org.objectweb.fractal.adl.bindings.Binding}
 * nodes in definitions. This loader checks that all mandatory client interfaces are bound.
 */

public class UnboundInterfaceChecker extends AbstractLoader {

    public Definition load (final String name, final Map context)
    throws ADLException
    {
        // Load the definition with the next loader.
        Definition d = clientLoader.load(name, context);

        // Check the loaded definition.
        checkDefinition(d);

        return d;
    }

    // --------------------------------------------------------------------------
    // Checking methods
    // --------------------------------------------------------------------------

    private void checkDefinition (final Definition definition)
    throws ADLException
    {
        // This map will store all the components having some unbound mandatory client interfaces.
        // The keys of the map are Component objects.
        // The values of the map are ArrayList objects containing TypeInterface_Node objects.
        Map unboundMandatoryClientInterfacesMap = new HashMap();

        // Check the definition and feed the map of unbound mandatory client interfaces.
        checkNode(definition, unboundMandatoryClientInterfacesMap);

        // Iterate on all the components having some unbound mandatory client interfaces.
        for(Iterator iterMap=unboundMandatoryClientInterfacesMap.entrySet().iterator();
            iterMap.hasNext(); ) {
            Map.Entry entry = (Map.Entry)iterMap.next();
            Component comp = (Component)entry.getKey();
            ArrayList list = (ArrayList)entry.getValue();
            for(Iterator iterList=list.iterator(); iterList.hasNext(); ) {
                TypeInterface_Node itfNode = (TypeInterface_Node)iterList.next();
		System.err.println("Warning: Mandatory client interface '"
                                   + comp.getName() + '.' + itfNode.itf.getName()
                                   + " is not bound (" + itfNode.node.astGetSource() + ")");
            }
        }
    }

    private void checkNode (final Object node, final Map unboundMandatoryClientInterfacesMap)
    throws ADLException
    {
        if(node instanceof ComponentContainer) {
            Component[] comps = ((ComponentContainer)node).getComponents();

            // We do this only for composite components.
            if(comps.length>0) {

                // Computation of bound client interfaces of the composite.
                HashSet bindingClientsSet = new HashSet();
                if(node instanceof BindingContainer) {
                    Binding[] bindings = ((BindingContainer)node).getBindings();
                    for(int i=0; i<bindings.length; i++) {
                        bindingClientsSet.add(bindings[i].getFrom());
                    }
                }

                // Verification of unbounded mandatory client interfaces of subcomponents.
                for(int i=0; i<comps.length; i++) {
                    Component comp = comps[i];
                    Node compNode = (Node)comp;
                    ArrayList unboundMandatoryClientInterfacesList = null;

                    // Check if this component was not already checked.
                    if(compNode.astGetDecoration("areMandatoryClientInterfacesChecked") == null) {
                        compNode.astSetDecoration("areMandatoryClientInterfacesChecked", "yes");
                        if(comp instanceof InterfaceContainer) {
                            Interface[] interfaces = ((InterfaceContainer)comp).getInterfaces();
                            for(int j=0; j<interfaces.length; j++) {
                                TypeInterface itf = (TypeInterface)interfaces[j];
                                // Check if it is a mandatory client interface.
                                if(  TypeInterface.CLIENT_ROLE.equals(itf.getRole())
                                  && isMandatoryInterface(itf)) {
                                    // In this case, this interface must be bound.
                                    if(!isBoundInterface(bindingClientsSet, comp.getName(), itf)) {
				        // The interface is not bound.
                                        if(unboundMandatoryClientInterfacesList == null) {
                                            // Create the list of unbound mandatory client interfaces for this component.
                                            unboundMandatoryClientInterfacesList = new ArrayList();
                                            // Add it to the global map.
                                            unboundMandatoryClientInterfacesMap.put(comp, unboundMandatoryClientInterfacesList);
                                        }
                                        // Add the unbound interface to the list.
                                        unboundMandatoryClientInterfacesList.add(new TypeInterface_Node(itf, (Node)node));
                                    }
                                 }
                            }
                        }
                    } else {
                        // In this case, the component was already checked.
                        unboundMandatoryClientInterfacesList = (ArrayList)unboundMandatoryClientInterfacesMap.get(comp);
                        if(unboundMandatoryClientInterfacesList != null) {
			    // Remove from this list the unbound mandatory client interfaces of this component
                            // which are bound in this composite.
                            for(Iterator iter=unboundMandatoryClientInterfacesList.iterator(); iter.hasNext(); ) {
                                TypeInterface_Node itfNode = (TypeInterface_Node)iter.next();
                                if(isBoundInterface(bindingClientsSet, comp.getName(), itfNode.itf)) {
                                    iter.remove();
                                }
                            } 
                            // Remove the component from the global map when its list is empty.
                            if(unboundMandatoryClientInterfacesList.size() == 0)
                                unboundMandatoryClientInterfacesMap.remove(comp);
                        }
                    }
                }

                // Verification of the unbound internal client interfaces
                // of this composite (applied only to composite components)
                if(node instanceof InterfaceContainer) {
                    Interface[] interfaces = ((InterfaceContainer)node).getInterfaces();
                    for(int i=0; i<interfaces.length; i++) {
                        TypeInterface itf = (TypeInterface)interfaces[i];
                        // Check if it is a mandatory server interface but
                        // not a controller interface.
                        if(   TypeInterface.SERVER_ROLE.equals(itf.getRole())
                           && isMandatoryInterface(itf)
                           && (!itf.getName().endsWith("controller"))) {
                            // In this case, this interface must be bounded
                            if(!isBoundInterface(bindingClientsSet, "this", itf)) {
                                String name = null;
                                if (node instanceof Definition) {
                                    name = ((Definition)node).getName();
                                } else if (node instanceof Component) {
                                    name = ((Component)node).getName();
                                }
                                throw new ADLException("Internal client interface of the server interface of composite \""
                                                       + name + "." + itf.getName() + "\" is not bound",
                                                       (Node)node);
                            }
                        }
                    }
                }
            }

            // Check all subcomponents recursively.
            for(int i=0; i<comps.length; i++) {
                checkNode(comps[i], unboundMandatoryClientInterfacesMap);
            }
        }
    }

    private boolean isBoundInterface (
      final HashSet presentBindings,
      final String prefix,
      final TypeInterface itf)
    {
        final String itfName = prefix + "." + itf.getName();

        if(isSingletonInterface(itf)) {
            return presentBindings.contains(itfName);
        }
        // This is a collection interface then it is bound if
        // there is at least one binding starting with the interface name.
        for(Iterator iter=presentBindings.iterator(); iter.hasNext(); ) {
            if(((String)iter.next()).startsWith(itfName))
                return true;
        }
        return false;
    }

    private boolean isMandatoryInterface (final TypeInterface itf)
    {
        // We consider that if the contingency of an interface is not defined then it is mandatory.
        final String contingency = itf.getContingency();
        return contingency == null || TypeInterface.MANDATORY_CONTINGENCY.equals(contingency);
    }

    private boolean isSingletonInterface (final TypeInterface itf)
    {
        // We consider that if the cardinality of an interface is not defined then it is singleton.
        final String cardinality = itf.getCardinality();
        return cardinality == null || TypeInterface.SINGLETON_CARDINALITY.equals(cardinality);
    }
}

/**
 * Helper class to store a {@link org.objectweb.fractal.adl.types.TypeInterface}
 * and a {@link org.objectweb.fractal.adl.Node} objects.
 */

class TypeInterface_Node {

  public final TypeInterface itf;

  public final Node node;

  public TypeInterface_Node (
    final TypeInterface itf,
    final Node node)
  {
    this.itf = itf;
    this.node = node;
  }
}
